# -*- coding: utf-8 -*-

# Copyright © 2018 by IBPort. All rights reserved.
# @Author: Neal Wong
# @Email: ibprnd@gmail.com

import json

from dropshipping.utils.mws_value_extracter import (
    get_string_value,
    get_boolean_value,
    get_non_negative_integer_value,
    get_non_negative_integer_with_units_value,
    get_decimal_with_units_value,
    get_dimension_value,
    get_language_value,
    get_price_value,
    get_image_value,
    get_creator_value,
    get_sales_rank_list_value
)

from dropshipping.mws import MARKETPLACE_COUNTRY_MAPPING


class ProductConverter(object):

    def convert(self, data):
        raise NotImplementedError()


class MwsMatchingProductConverter(ProductConverter):

    def convert(self, data):
        result = dict()

        matching_product_result_list = data.parsed
        if not isinstance(matching_product_result_list, list):
            matching_product_result_list = [matching_product_result_list]

        d = dict()
        for matching_product in matching_product_result_list:
            asin = matching_product.get('ASIN', d).get('value', None)
            if asin is None:
                continue

            marketplace = (
                matching_product.get('Product', d)
                .get('Identifiers', d)
                .get('MarketplaceASIN', d)
                .get('MarketplaceId', d)
                .get('value', None))
            country = MARKETPLACE_COUNTRY_MAPPING.get(marketplace, None)
            if country is None:
                continue

            product = matching_product.get('Product', d)

            item_attributes = dict()
            item_attributes_d = product.get('AttributeSets', d).get('ItemAttributes', d)
            item_attributes_fields = {
                'string': [
                    'Actor', 'Artist', 'AspectRatio', 'AudienceRating', 'Author', 'BackFinding',
                    'BandMaterialType', 'Binding', 'BlurayRegion', 'Brand', 'CEROAgeRating',
                    'ChainType', 'ClaspType', 'Color', 'CPUManufacturer', 'CPUType', 'Department',
                    'Director', 'Edition', 'EpisodeSequence', 'ESRBAgeRating', 'Feature',
                    'Flavor', 'Format', 'GemType', 'Genre', 'GolfClubFlex', 'HandOrientation',
                    'HardDiskInterface', 'HardwarePlatform', 'HazardousMaterialType',
                    'IssuesPerYear', 'ItemPartNumber', 'Label', 'LegalDisclaimer', 'Manufacturer',
                    'ManufacturerPartsWarrantyDescription', 'MaterialType', 'MediaType',
                    'MetalStamp', 'MetalType', 'Model', 'OperatingSystem', 'PartNumber',
                    'PegiRating', 'Platform', 'ProductGroup', 'ProductTypeName',
                    'ProductTypeSubcategory', 'PublicationDate', 'Publisher', 'RegionCode',
                    'ReleaseDate', 'RingSize', 'ShaftMaterial', 'Scent', 'SeasonSequence',
                    'SeikodoProductCode', 'Size', 'SizePerPearl', 'Studio', 'SystemMemoryType',
                    'TheatricalReleaseDate', 'Title', 'Warranty'
                ],
                'boolean': [
                    'IsAdultProduct', 'IsAutographed', 'IsEligibleForTradeIn', 'IsMemorabilia'
                ],
                'DecimalWithUnits': [
                    'CPUSpeed', 'DisplaySize', 'GolfClubLoft', 'HardDiskSize',
                    'ManufacturerMaximumAge', 'ManufacturerMinimumAge', 'MaximumResolution',
                    'OpticalZoom', 'RunningTime', 'SystemMemorySize', 'TotalDiamondWeight',
                    'TotalGemWeight'
                ],
                'CreatorType': [
                    'Creator'
                ],
                'DimensionType': [
                    'ItemDimensions', 'PackageDimensions'
                ],
                'LanguageType': [
                    'Languages'
                ],
                'Price': [
                    'ListPrice', 'WEEETaxValue'
                ],
                'nonNegativeInteger': [
                    'NumberOfDiscs', 'NumberOfIssues', 'NumberOfItems', 'NumberOfPages',
                    'NumberOfTracks', 'PackageQuantity', 'ProcessorCount', ''
                ],
                'NonNegativeIntegerWithUnits': ['SubscriptionLength'],
                'Image': ['SmallImage']
            }
            for t, fields in item_attributes_fields.items():
                if t == 'string':
                    func = get_string_value
                elif t == 'boolean':
                    func = get_boolean_value
                elif t == 'DecimalWithUnits':
                    func = get_decimal_with_units_value
                elif t == 'CreatorType':
                    func = get_creator_value
                elif t == 'DimensionType':
                    func = get_dimension_value
                elif t == 'LanguageType':
                    func = get_language_value
                elif t == 'Price':
                    func = get_price_value
                elif t == 'nonNegativeInteger':
                    func = get_non_negative_integer_value
                elif t == 'NonNegativeIntegerWithUnits':
                    func = get_non_negative_integer_with_units_value
                elif t == 'Image':
                    func = get_image_value
                else:
                    continue

                for field in fields:
                    item_attributes[field] = func(item_attributes_d.get(field, None))

            sales_ranks = get_sales_rank_list_value(product.get('SalesRankings', d))

            result[asin] = {
                'ItemAttributes': item_attributes,
                'SalesRankings': sales_ranks
            }

        return result

class EsMatchingProductConverter(ProductConverter):

    def convert(self, data):
        result = dict()

        if data:
            for item in data.get('hits', {}).get('hits', []):
                result[item['_id']] = json.loads(item['_source']['matching_product'])
        else:
            result = data

        return result


class DvEsMatchingProductConverter(ProductConverter):

    def convert(self, data):
        result = dict()

        if data:
            for item in data.get('hits', {}).get('hits', []):
                result[item['_id']] = item['_source']
        else:
            result = data

        return result
